@@ -443,10 +443,7 @@ pub(crate) fn generate(args: DeriveInput) -> GeneratorResult<TokenStream> {
     let from_request = args.generate_from_request(&crate_name);
     let checker = args.checker.as_ref().map(|path| {
         quote! {
-            let output = {
-                let res: ::std::result::Result<_, #crate_name::__private::poem::Error> = #path(&req, output).await;
-                res?
-            };
+            let output = ::std::option::Option::ok_or(#path(&req, output).await, #crate_name::error::AuthorizationError)?;
         }
     });
 
